from django.contrib import admin

from .models import *

class SocialShareLinkInline(admin.TabularInline):
    model = SocialShareLink 
    extra = 0
    sortable_field_name = "order"

class SocialShareSettingsAdmin(admin.ModelAdmin):
    inlines = [SocialShareLinkInline]

    list_display = ('title', 'site',)
    readonly_fields = ('title',)
    list_filter = ('site',)

class SocialShareTrackAdmin(admin.ModelAdmin):

    list_display = ('created', 'full_url', 'domain', 'path', 'type')
    list_filter = ('full_url', 'domain', 'path', 'type', 'created')
    search_fields = ('full_url','domain','path', 'type')
    fields = ['domain','path','full_url','type','created']
    csv_fields = ['domain','path','full_url','type','created']
    readonly_fields = ('created',)

admin.site.register(SocialShareSettings, SocialShareSettingsAdmin)
admin.site.register(SocialShareTrack, SocialShareTrackAdmin)


import urllib

from django.db import models
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.sites.models import Site
from django.template import Template, Context





class SocialShareSettings( models.Model ):

    site = models.ForeignKey(Site, null=True, blank=True)

    track_social_share_clicks = models.BooleanField( _("Track Social Share Clicks"), default = True )
    
    def get_social_share_links(self):
        return SocialShareLink.objects.filter(parent=self).order_by('order')

    @staticmethod
    def get_site_settings():
        current_site = Site.objects.get_current()
        try:
            return SocialShareSettings.objects.filter(site=current_site)[0]
        except:
            try:
                return SocialShareSettings.objects.all()[0]
            except:
                return None

    @property
    def title(self):
        if self.site:
            return 'Social Share Settings %s'%(self.site.domain)
        return "Default Social Share Settings"

    class Meta:
        verbose_name_plural = _("Social Share Settings")

    def __unicode__(self):
        return self.title
     

# class SocialShareTrack( models.Model ):

#     created = models.DateTimeField(_('Created Date'), auto_now_add=True, 
#         blank=True, null=True)

#     service = models.CharField()
#     url = models.CharField()
#     ipaddress = models.CharField()


class SocialShareLink( models.Model ):

    parent = models.ForeignKey(SocialShareSettings)
    order = models.PositiveIntegerField('order', null = True, blank=True)


    TYPE_EMAIL = 'email'
    TYPE_TWITTER = 'twitter'
    TYPE_FACEBOOK = 'facebook'
    TYPE_GOOGLEPLUS = 'googleplus'
    TYPE_LINKEDIN = 'linkedin'
    TYPE_PINTEREST = 'pinterest'
    TYPE_DIGG = 'digg'
    TYPE_TUMBLR = 'tumblr'
    TYPE_REDDIT = 'reddit'
    TYPE_STUMBLEUPON = 'stumbleupon'
    TYPE_DELICIOUS = 'delicious'
    

    SERVICE_TYPES = (
        (TYPE_EMAIL, "Email"),
        (TYPE_TWITTER, "Twitter"),
        (TYPE_FACEBOOK, "Facebook"),
        (TYPE_GOOGLEPLUS, "Google Plus"),
        (TYPE_LINKEDIN, "LinkedIn"),
        (TYPE_PINTEREST, "Pinterest"),
        (TYPE_DIGG, "Digg"),
        (TYPE_TUMBLR, "Tumblr"),
        (TYPE_REDDIT, "Reddit"),
        (TYPE_STUMBLEUPON, "StumbleUpon"),
        (TYPE_DELICIOUS, "Delicious"),
    )

    FONT_AWESOME_CLASSES = {
        TYPE_EMAIL: "envelope",
        TYPE_TWITTER: "twitter",
        TYPE_FACEBOOK: "facebook",
        TYPE_GOOGLEPLUS: "google-plus",
        TYPE_LINKEDIN: "linkedin",
        TYPE_PINTEREST: "pinterest",
        TYPE_DIGG: "digg",
        TYPE_TUMBLR: "tumblr",
        TYPE_REDDIT: "reddit",
        TYPE_STUMBLEUPON: "stumbleupon",
        TYPE_DELICIOUS: "delicious",
    }


    type = models.CharField("Service Type", max_length=255, 
        null=True, blank=True, choices=SERVICE_TYPES)

    title_template = models.TextField(_("Title Template"), blank=True, null=True, help_text="Applicable to all types but Facebook and Google Plus. Available contact variables: {{url}}, {{title}}, {{site}}")
    description_template = models.TextField(_("Description Template"), blank=True, null=True, help_text="Applicable to type Email and Tumblr. Available contact variables: {{url}}, {{title}}, {{site}}")
    to_template = models.TextField(_("To Template"), blank=True, null=True, help_text="Applicable to type Email. Available contact variables: {{url}}, {{title}}, {{site}}")

    @property
    def font_awesome_class(self):
        return SocialShareLink.FONT_AWESOME_CLASSES[self.type]

    def get_share_url(self, full_path, object_title):

        if self.parent.track_social_share_clicks and self.type != SocialShareLink.TYPE_EMAIL:
            return self._get_track_url(full_path, object_title)
        else:
            return self._get_share_url(full_path, object_title)

    def _get_track_url(self, full_path, object_title):

        path = reverse('share_counter_redirect_view')
        
        params = { 
            'url': full_path,
            'type': self.type,
            'title': object_title
        }
        track_url = '%s?%s' % (path, urllib.urlencode(params))

        return track_url

    def _get_share_url(self, full_path, object_title):
        if self.type == SocialShareLink.TYPE_EMAIL:
            return self.get_email_share_url(full_path, object_title)
            
        elif self.type == SocialShareLink.TYPE_TWITTER:
            return self.get_twitter_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_FACEBOOK:
            return self.get_facebook_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_GOOGLEPLUS:
            return self.get_googleplus_share_url(full_path, object_title)
            
        elif self.type == SocialShareLink.TYPE_LINKEDIN:
            return self.get_linkedin_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_PINTEREST:
            return self.get_pinterest_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_DIGG:
            return self.get_digg_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_TUMBLR:
            return self.get_tumblr_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_REDDIT:
            return self.get_reddit_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_STUMBLEUPON:
            return self.get_stumbleupon_share_url(full_path, object_title)

        elif self.type == SocialShareLink.TYPE_DELICIOUS:
            return self.get_delicious_share_url(full_path, object_title)
        

    def get_email_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'mailto:'
        
        to_rendered = self.get_rendered_content(self.to_template, page_url, page_title, site) if self.to_template else ''
        
        #body_cleaned = body.replace("$linebreak", "%0D%0A")
        body_cleaned = urllib.quote_plus(self.get_rendered_content(self.description_template, page_url, page_title, site))
        body_cleaned = body_cleaned.replace("+", "%20")
        

        #subject_cleaned = subject.replace("|", "%7C")
        subject_cleaned = urllib.quote_plus(self.get_rendered_content(self.title_template, page_url, page_title, site),)
        subject_cleaned = subject_cleaned.replace("+", "%20")

        return '%s%s?body=%s&subject=%s' % (query_root, to_rendered, body_cleaned, subject_cleaned)
        

    def get_twitter_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://twitter.com/intent/tweet?'
        params = { 
            'text': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_facebook_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://www.facebook.com/sharer.php?'
        params = { 
            'u': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_googleplus_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://plus.google.com/share?'
        params = { 
            'url': page_url
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_linkedin_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://www.linkedin.com/shareArticle?'
        params = { 
            'url': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_pinterest_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://pinterest.com/pin/create/bookmarklet/?'
        params = { 
            'url': page_url,
            'description': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_digg_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://digg.com/submit?'
        params = { 
            'url': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_tumblr_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://www.tumblr.com/share/link?'
        params = { 
            'url': page_url,
            'name': self.get_rendered_content(self.title_template, page_url, page_title, site),
            'description': self.get_rendered_content(self.description_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_reddit_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://reddit.com/submit?'
        params = { 
            'url': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_stumbleupon_share_url(self, page_url, page_title):
        site = Site.objects.get_current()
        query_root = 'https://www.stumbleupon.com/submit?'
        params = { 
            'url': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_delicious_share_url(self, page_url, page_title):

        site = Site.objects.get_current()
        query_root = 'https://delicious.com/save?v=5&provider=CCL&noui&jump=close&'
        params = { 
            'url': page_url,
            'title': self.get_rendered_content(self.title_template, page_url, page_title, site)
        }
        return '%s%s' % (query_root, urllib.urlencode(params))

    def get_rendered_content(self, template_content, page_url, page_title, site):
        if not template_content:
            return "%s | %s"%(page_title, page_url)
        template = Template(template_content)
        context = Context({
            "url": page_url,
            "title": page_title,
            "site": site
        })
        return template.render(context)


class SocialShareTrack( models.Model ):

    domain = models.CharField(_("Domain"), blank=True, null=True, max_length=255,)
    path = models.CharField(_("Path"), blank=True, null=True, max_length=255,)
    full_url = models.CharField(_("Full URL"), blank=True, null=True, max_length=255,)

    created = models.DateTimeField(_('Created Date'), auto_now_add=True, 
        blank=True, null=True)

    type = models.CharField("Service Type", max_length=255, 
        null=True, blank=True, choices=SocialShareLink.SERVICE_TYPES)
    
    @staticmethod
    def get_count(type=None, from_datetime=None, to_datetime=None, domain=None, path=None, full_url=None ):
        all_tracks = SocialShareTrack.objects.all()
        if type:
            all_tracks = all_tracks.filter(type=type)

        if from_datetime:
            all_tracks = all_tracks.filter(created__gte=from_datetime)

        if to_datetime:
            all_tracks = all_tracks.filter(created__lte=to_datetime)

        if domain:
            all_tracks = all_tracks.filter(domain=domain)

        if path:
            all_tracks = all_tracks.filter(path=path)

        if full_url:
            all_tracks = all_tracks.filter(full_url=full_url)

        return all_tracks

    @classmethod
    def track(cls, domain, path, full_url, type):
        track = cls(
            domain=domain,
            path=path,
            full_url=full_url,
            type=type
        )
        track.save()
        return track



from django.conf.urls.static import static
from django.conf.urls import include, url


from .views import *

urlpatterns = [
    
    url(r'^track/shares/$', ShareCounterRedirectView.as_view(), name="share_counter_redirect_view"),
    
    url(r'^admin/dashboard_stats/share_me_share_me/socialsharetrack/$', DashboardStatsView.as_view(), name='dashboard_stats_share_me_share_me'),
    url(r'^admin/dashboard_stats/share_me_share_me/socialsharetrack/paths/$', DashboardMostSharedPathsStatsView.as_view(), name='dashboard_stats_paths_share_me_share_me'),
    url(r'^admin/dashboard_stats/share_me_share_me/socialsharetrack/services/$', DashboardMostSharedServicesStatsView.as_view(), name='dashboard_stats_services_share_me_share_me'),
]


from urlparse import urlparse
import datetime

from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator
from django.views.generic.base import RedirectView
from django.views.generic import ListView

from .models import *

DEFAULT_DURATION_DAYS = 30

class ShareCounterRedirectView(RedirectView):

    permanent = False
    query_string = True
    
    def get_redirect_url(self, *args, **kwargs):


        full_url = self.request.GET.get('url', None)
        if full_url == None:
            return None

        type = self.request.GET.get('type', None)
        if type == None:
            return None

        title = self.request.GET.get('title', None)
        
        #Retreieve settings
        settings = SocialShareSettings.get_site_settings()
        link_settings = SocialShareLink.objects.get(type=type,parent=settings)



        parsed_uri = urlparse( full_url )
        domain = '{uri.scheme}://{uri.netloc}/'.format(uri=parsed_uri)

        #Store 
        SocialShareTrack.track(domain, parsed_uri.path, full_url, type)

        #return full_url
        share_url = link_settings._get_share_url(full_url, title)

        return share_url

##############################
## ADMIN VIEWS ###############
##############################

def get_latest_shared_paths(queryset, duration_days):
    
    filter_duration = datetime.datetime.now() - datetime.timedelta(days=duration_days)
    queryset = queryset.filter(created__gte=filter_duration)

    path_querset = queryset.values('path').annotate(total=models.Count('path')).order_by('-total')
    
    shared_paths = {
        'title':"Most Shared Paths (Last %s days)"%(duration_days),
        'slug':'most-shared-urls',
        'links':path_querset[:5],
        'more_link_title':"See More",
        'more_link':"%s?%s"%(reverse('dashboard_stats_paths_share_me_share_me'), 'days=365'),
        'display_stat':'path',
        'duration_days':duration_days
    }
    return shared_paths

def get_latest_shared_services(queryset, duration_days):
    
    filter_duration = datetime.datetime.now() - datetime.timedelta(days=duration_days)
    queryset = queryset.filter(created__gte=filter_duration)

    service_queryset = queryset.values('type').annotate(total=models.Count('type')).order_by('-total')

    shared_services = {
        'title':"Most Shared Services (Last %s days)"%(duration_days),
        'slug':'most-shared-networks',
        'links':service_queryset[:5],
        'more_link_title':"See More",
        'more_link':"%s?%s"%(reverse('dashboard_stats_services_share_me_share_me'), 'days=365'),
        'display_stat':'type',
        'duration_days':duration_days
    }
    return shared_services    

class DashboardStatsView(ListView):

    model = SocialShareTrack
    template_name = "admin/share_me_share_me/socialsharetrack/dashboard_stats.html"

    @method_decorator(user_passes_test(lambda u: u.is_staff))
    def dispatch(self, *args, **kwargs):
        return super(DashboardStatsView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        queryset = self.model._default_manager.all()

        duration_days = int(self.request.REQUEST.get('days', DEFAULT_DURATION_DAYS))
        
        self.shared_paths = get_latest_shared_paths(queryset, duration_days)

        self.shared_services = get_latest_shared_services(queryset, duration_days)

        return [self.shared_paths, self.shared_services]


class DashboardMostSharedPathsStatsView(ListView):

    model = SocialShareTrack
    template_name = "admin/share_me_share_me/socialsharetrack/dashboard_most_shared_paths_stats.html"

    @method_decorator(user_passes_test(lambda u: u.is_staff))
    def dispatch(self, *args, **kwargs):
        return super(DashboardMostSharedPathsStatsView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        queryset = self.model._default_manager.all()

        self.duration_days = int(self.request.REQUEST.get('days', DEFAULT_DURATION_DAYS))
        
        self.shared_paths = get_latest_shared_paths(queryset, self.duration_days)

        return self.shared_paths['links']


    def get_context_data(self, **kwargs):
        ctx = super(DashboardMostSharedPathsStatsView, self).get_context_data(**kwargs)
        ctx['title'] = self.shared_paths['title']
        ctx['slug'] = self.shared_paths['slug']
        ctx['display_stat'] = self.shared_paths['display_stat']
        ctx['duration_days'] = self.shared_paths['duration_days']
        return ctx


class DashboardMostSharedServicesStatsView(ListView):

    model = SocialShareTrack
    template_name = "admin/share_me_share_me/socialsharetrack/dashboard_most_shared_services_stats.html"

    @method_decorator(user_passes_test(lambda u: u.is_staff))
    def dispatch(self, *args, **kwargs):
        return super(DashboardMostSharedServicesStatsView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        queryset = self.model._default_manager.all()

        self.duration_days = int(self.request.REQUEST.get('days', DEFAULT_DURATION_DAYS))
        
        self.shared_services = get_latest_shared_services(queryset, self.duration_days)

        return self.shared_services['links']  

    def get_context_data(self, **kwargs):
        ctx = super(DashboardMostSharedServicesStatsView, self).get_context_data(**kwargs)
        ctx['title'] = self.shared_services['title']
        ctx['slug'] = self.shared_services['slug']
        ctx['display_stat'] = self.shared_services['display_stat']
        ctx['duration_days'] = self.shared_services['duration_days']
        return ctx




try:
    from site_admin.dashboard import dashboard_registry, DashboardLoaderModuleRegistry
    dashboard_registry.register(DashboardLoaderModuleRegistry.CATEGORY_COLUMN_THREE, 'Share Stats', 'dashboard_stats_share_me_share_me')
except:
    pass
    #not implemented


from setuptools import setup, find_packages
#this is a test
setup(name = 'django-share-me-share-me',
      description = 'Model and front end library for choosing and integrating social media share widgets.',
      version = '1.3',
      url = 'https://github.com/ninapavlich/django-share-me-share-me',
      author = 'Nina Pavlich',
      author_email='nina@ninalp.com',
      license = 'BSD',
      packages=find_packages(exclude=['ez_setup']),
      zip_safe = False,
      include_package_data=True,
      install_requires = ['setuptools', 'Django'],
      classifiers=[
       'Development Status :: 4 - Beta',
       'Environment :: Web Environment',
       'Framework :: Django',
       'Intended Audience :: Developers',
       'License :: OSI Approved',
       'Operating System :: OS Independent',
       'Programming Language :: Python'
      ]
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SocialShareLink',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('order', models.PositiveIntegerField(null=True, verbose_name=b'order', blank=True)),
                ('type', models.CharField(blank=True, max_length=255, null=True, verbose_name=b'Service Type', choices=[(b'email', b'Email'), (b'twitter', b'Twitter'), (b'facebook', b'Facebook'), (b'googleplus', b'Google Plus'), (b'linkedin', b'LinkedIn'), (b'pinterest', b'Pinterest'), (b'digg', b'Digg'), (b'tumblr', b'Tumblr'), (b'reddit', b'Reddit'), (b'stumbleupon', b'StumbleUpon'), (b'delicious', b'Delicious')])),
                ('title_template', models.TextField(help_text=b'Applicable to all types but Facebook and Google Plus. Available contact variables: {{url}}, {{title}}, {{site}}', null=True, verbose_name='Title Template', blank=True)),
                ('description_template', models.TextField(help_text=b'Applicable to type Email and Tumblr. Available contact variables: {{url}}, {{title}}, {{site}}', null=True, verbose_name='Description Template', blank=True)),
                ('to_template', models.TextField(help_text=b'Applicable to type Email. Available contact variables: {{url}}, {{title}}, {{site}}', null=True, verbose_name='To Template', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SocialShareSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('track_social_share_clicks', models.BooleanField(default=True, verbose_name='Track Social Share Clicks')),
                ('site', models.ForeignKey(blank=True, to='sites.Site', null=True)),
            ],
            options={
                'verbose_name_plural': 'Social Share Settings',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SocialShareTrack',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('domain', models.CharField(max_length=255, null=True, verbose_name='Domain', blank=True)),
                ('path', models.CharField(max_length=255, null=True, verbose_name='Path', blank=True)),
                ('full_url', models.CharField(max_length=255, null=True, verbose_name='Full URL', blank=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created Date', null=True)),
                ('type', models.CharField(blank=True, max_length=255, null=True, verbose_name=b'Service Type', choices=[(b'email', b'Email'), (b'twitter', b'Twitter'), (b'facebook', b'Facebook'), (b'googleplus', b'Google Plus'), (b'linkedin', b'LinkedIn'), (b'pinterest', b'Pinterest'), (b'digg', b'Digg'), (b'tumblr', b'Tumblr'), (b'reddit', b'Reddit'), (b'stumbleupon', b'StumbleUpon'), (b'delicious', b'Delicious')])),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='socialsharelink',
            name='parent',
            field=models.ForeignKey(to='share_me_share_me.SocialShareSettings'),
            preserve_default=True,
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from django.template import Library
register = Library()

from ..models import *

@register.assignment_tag(takes_context=True)
def get_social_share_links(context):

    settings = SocialShareSettings.get_site_settings()
    if settings:
        return settings.get_social_share_links()
    return []
    
@register.assignment_tag(takes_context=True)
def get_social_share_link(context, share_link, object_url, object_title):

    """
    Construct the social share link for the request object.

    """
    request = context['request']
    url = unicode(object_url)
    if 'http' not in object_url.lower():
        full_path = ''.join(('http', ('', 's')[request.is_secure()], '://', request.META['HTTP_HOST'], url))
    else:
        full_path = url

    return share_link.get_share_url(full_path, object_title)

@register.assignment_tag(takes_context=True)
def get_social_share_count(context, type):

    return SocialShareTrack.get_count(type)   
        

@register.assignment_tag()
def item_admin_link(object, attr):
    link = reverse('admin:share_me_share_me_socialsharetrack_changelist')
    return '%s?q=%s'%(link, object[attr])


@register.assignment_tag()
def get_stats_display(object, attr):
    return '%s (%s)'%(object[attr], object['total'])